%% Box Particle Filter for multiple extended objects
%% doi:L10.1109/TAES.2018.2874147
%% Code written by: Allan De Freitas (allan.defreitas@tuks.co.za)
%% Requires Intlab Toolbox (http://www.ti3.tu-harburg.de/rump/intlab/) 
%% for interval operations

close all; clear;

if ~exist('cont','var')
    try
        run(cat(2,pwd,'/Intlab_V7.1/startup.m'))
    catch
        error('Change the path to point to the IntLab startup.m file')
    end
end

f1 = figure(1);
initialisation

load('sensor_measurements.mat')

estAll = zeros(20,length(laser)-1);
estNT = zeros(length(laser)-1);

k = 1;
while k <= length(laser)
    hold off
    plotCorridor
    xlabel('Particles','FontSize', 13) 
    grd(1).numSen = 3;
    grd(2).numSen = 3;
    grd(3).numSen = 3;
    grd(1).Xsen = -1.55;
    grd(3).Xsen = 1.54;
    grd(2).Xsen = 0;
    grd(1).Ysen = 0;
    grd(3).Ysen = 0;
    grd(2).Ysen = -5.17;

    for s = 1:grd(1).numSen
        meas(s).d = laser(k).rho(s,laser(k).in(s,:));
        meas(s).b = laser(k).theta(s,laser(k).in(s,:));
        meas(s).d = meas(s).d + infsup(-param.distanceMeasurementBox,param.distanceMeasurementBox);
        meas(s).b = meas(s).b + infsup(-param.angleMeasurementBox,param.angleMeasurementBox);
        param.numMeas = size(meas(s).d.inf,2);
        param.numRealMeas = param.numMeas;
        param1(s) = param;
        clutterColour = {'.b' '.c' '.k'}; 
        if sum(laser(k).in(s,:))
            clutter1(k,s) = plot(laser(k).data(s*2-1,laser(k).in(s,:)), laser(k).data(s*2,laser(k).in(s,:)),clutterColour{s});
        end  
    end
    
    if k == 1
        [particles(1:5,:),weights] = generateParticles(particles(1:5,1), param);
    else
        indx = rand(max_targets,param.numParticles) > param.gamma;
        existTOld = existT;
        existT(indx) = ~existT(indx);
        Birth = zeros(max_targets,param.numParticles);
        Birth((existT==1) & (existTOld ==0)) = 1;
        for Nm = 1:max_targets
            particlesProp((5*(Nm-1))+1:5*Nm,:) = inputFunctionEst(particles((5*(Nm-1))+1:5*Nm,:), param, existTOld(Nm,:), existT(Nm,:)); %inputFunctionEst(particles, param);
        end
    end
    proc
    estAll(:,k) = estimate;
    estNT(k) = sum(estExistT);
    
    for s2 = 1:grd(1).numSen
        plot(laser(k).data(2*s2 - 1,laser(k).in(s2,:)), laser(k).data(2*s2,laser(k).in(s2,:)),'r.')
    end
    
    for Nm=1:max_targets
        if estExistT(Nm)
            ellipse(estimate(Nm*5),estimate(Nm*5),0,estimate((Nm-1)*5+1),estimate((Nm-1)*5+3),'g',360,1);
        end
    end
    drawnow
    
    fprintf('%.2f %% Complete \n', 100*((k-1+1)/(length(laser)-1)))
    k = k + 1;
    
end



